#!/usr/bin/env python3
#
# Copyright (c) 2018, Pierre-Elliott Bécue
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
"""
This is a rough binary command that casts a converter and calls
convert on the user's input.
"""

import argparse

import flatlatex


def do_convert(arguments):
    """Do the actual conversion"""

    converter = flatlatex.converter(
        arguments.disallow_zero_width,
        arguments.disallow_combinings,
    )

    return converter.convert(arguments.input[0])


def main():
    parser = argparse.ArgumentParser(
        description="Convert latex entries to flat unicode"
    )
    parser.add_argument('-C', '--disallow-combinings',
                        help="Disallow combining characters to be used",
                        action="store_false",
                        default=True)
    parser.add_argument('-Z', '--disallow-zero-width',
                        help="Disallow zero-width characters to be used",
                        action="store_false",
                        default=True)
    parser.add_argument("input", type=str, nargs=1,
                        help="The latex string to convert")

    args = parser.parse_args()

    ret = do_convert(args)
    print(ret)



# Copyright (c) 2017-2018, Jean-Benoist Leger <jb@leger.tf>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.


from .. import parser


def test_parser():

    expected = {
            r'\\': [('cmd',  r'\\')],
            r'\cd': [('cmd',  r'\cd')],
            r'ab': [('char', 'a'), ('char', 'b')],
            r'e{to{t}o}e': [
                ('char', 'e'), ('subexpr', 'to{t}o'), ('char', 'e')
            ],
            r'g_f': [('char', 'g'), ('oper', '_'), ('char', 'f')],
            r'h^i': [('char', 'h'), ('oper', '^'), ('char', 'i')],
            }

    for k, v in expected.items():
        assert all([
            pki == vi
            for pki, vi in zip(parser.parse(k), v)
        ]), "parser '%s'" % k

    for k1, v1 in expected.items():
        for k2, v2 in expected.items():
            assert all([
                pki == vi
                for pki, vi in zip(parser.parse(k1 + ' ' + k2), v1 + v2)
            ]), "parser '%s'+'%s'" % (k1, k2)



# Copyright (c) 2016, Jean-Benoist Leger <jb@leger.tf>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from . import latexfuntypes


def transliterate(alphabet):
    alphabet_keys = alphabet.keys()

    def fun(flat_input):
        flat_output = ''
        success = True
        for c in flat_input:
            if c in alphabet_keys:
                flat_output += alphabet[c]
            else:
                flat_output += c
                success = False
        return (flat_output, success)
    return fun


def transliterator(alphabet):
    return latexfuntypes.latexfun(
        lambda x: transliterate(alphabet)(x[0])[0], 1
    )



# Copyright (c) 2016, Jean-Benoist Leger <jb@leger.tf>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import regex
from .latexfuntypes import LatexSyntaxError


def parse_one_element(s):
    R = r'((?>\\(?:[^A-Za-z]|[A-Za-z]+))|(?>[^\{\}\\])|\{(?1)*\})'
    r = regex.match(R, s)
    if not r:
        raise LatexSyntaxError
    s = s[r.span()[1]:]
    c = r.captures()[0]
    if c[0] == '\\':
        return (('cmd', c), s)
    if c[0] == '{':
        return (('subexpr', c[1:-1]), s)
    if c in ('_', '^'):
        return (('oper', c), s)
    return (('char', c), s)


def parse(s):
    ret = []
    while len(s) > 0:
        m, s = parse_one_element(s)
        if not (m[1] == ' '):
            ret.append(m)
    return ret



# Copyright (c) 2016-2018, Jean-Benoist Leger <jb@leger.tf>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.


from .. import converter

def test_conv1():
    c = converter()
    r = c.convert(
        (
            r'\forall \eta>0\, \exists n\in\mathbb{N}\, \forall i>n\, '
            r'|u_i-\mathcal{l}|<\eta'
        )
    )
    assert r == '∀η>0 ∃n∈ℕ ∀i>n |uᵢ-𝓵|<η'


def test_conv2():
    c = converter()
    c.add_newcommand(r'\newcommand\prob{\mathbb{P}}')
    c.add_newcommand(r'\newcommand\binom[2]{\frac{#2!}{#1!(#2-#1)!}}')
    r = c.convert(r'\prob(X=k)\,=\,\binom{k}{n}\times p^k(1-p)^{n-k}')
    assert r == 'ℙ(X=k) = (n!)/(k!(n-k)!)×pᵏ(1-p)ⁿ⁻ᵏ'


def test_conv3():
    c = converter()
    c.allow_zw = True
    r = c.convert(r'\frac{8}{9}')
    assert r == '⁸⁄₉'
    c.allow_zw = False
    r = c.convert(r'\frac{8}{9}')
    assert r == '8/9'


def test_conv4():
    c = converter()
    c.allow_combinings = True
    r = c.convert(r'\hat\alpha')
    assert r == '\u03B1\u0302'
    c.allow_combinings = False
    r = c.convert(r'\hat\alpha')
    assert r == 'hat(\u03B1)'



# Copyright (c) 2016, Jean-Benoist Leger <jb@leger.tf>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from . import data
from .transliteration import transliterator, transliterate
import regex
from . import latexfuntypes
from . import parser

from .latexfuntypes import LatexSyntaxError


class converter:
    """flatlatex converter class

    :attrib allow_zw: boolean which indicate if zero width characters are
        allowed (True by default).
    :attrib allow_combinings: boolean which indicate if combining characters
        are allowed (True by default).
    """

    def __init__(self, allow_zw=True, allow_combinings=True):
        """Initialize a convert method."""
        self.__cmds = {}

        # transliteration
        for k, v in data.transliterators.items():
            self.__cmds[k] = transliterator(v)

        # symbols
        def makefun(symb):
            return lambda x: symb
        for cmd in data.symbols:
            self.__cmds[cmd] = latexfuntypes.latexfun(
                makefun(data.symbols[cmd]),
                0)

        # combinings
        def makefun(comb):
            return lambda x: self.__latexfun_comb(comb, x)
        for cmd in data.combinings:
            self.__cmds[cmd] = latexfuntypes.latexfun(
                makefun(data.combinings[cmd]),
                1)

        # others
        self.__cmds[r'\frac'] = latexfuntypes.latexfun(self.__latexfun_frac, 2)
        self.__cmds[r'\sqrt'] = latexfuntypes.latexfun(self.__latexfun_sqrt, 1)

        # newcommands
        for nc in data.newcommands:
            self.add_newcommand(nc)

        # config section
        self.allow_zw = allow_zw
        self.allow_combinings = allow_combinings

    def convert(self, expr):
        """Convert LaTeX math to Unicode text.

        :param expr: LaTeX math expression to convert"""

        parsed = parser.parse(expr)
        outvec = []
        idx = 0
        while(idx < len(parsed)):
            element = parsed[idx]
            if element[0] == 'oper':
                outvec.append(('oper', element[1]))
                idx += 1
                continue
            if element[0] == 'char':
                outvec.append(('char', element[1]))
                idx += 1
                continue
            if element[0] == 'cmd':
                try:
                    pycmd = self.__cmds[element[1]]
                except KeyError:
                    outvec.append(('char', element[1]))
                    idx += 1
                    continue
                if len(parsed) <= idx + pycmd.nargs:
                    raise LatexSyntaxError
                args = [
                    self.convert(parsed[idx + k + 1][1])
                    for k in range(pycmd.nargs)
                ]
                outvec.append(('char', pycmd.fun(args)))
                idx += 1 + pycmd.nargs
                continue
            if element[0] == 'subexpr':
                outvec.append(('char', self.convert(element[1])))
                idx += 1
                continue
            raise Exception
        # subandsuperscript
        for idx in range(len(outvec)):
            if outvec[idx][0] == 'oper':
                if len(outvec) <= idx + 1:
                    raise LatexSyntaxError
                if outvec[idx + 1][0] == 'oper':
                    raise LatexSyntaxError
        for idx in range(len(outvec)):
            if outvec[idx][0] == 'oper' and outvec[idx][1] == '^':
                if idx + 2 < len(outvec):
                    if (outvec[idx + 2][0] == 'oper' and
                            outvec[idx + 2][1] == '_'):
                        # we invert ^ and _
                        (
                            outvec[idx],
                            outvec[idx + 1],
                            outvec[idx + 2],
                            outvec[idx + 3],
                        ) = (
                            outvec[idx + 2],
                            outvec[idx + 3],
                            outvec[idx],
                            outvec[idx + 1],
                        )
        # sub
        newoutvec = []
        idx = 0
        while idx < len(outvec):
            if idx + 1 < len(outvec):
                if outvec[idx + 1][0] == 'oper' and outvec[idx + 1][1] == '_':
                    newoutvec.append(
                        (
                            'char',
                            self.__indexed(
                                outvec[idx][1],
                                outvec[idx + 2][1]
                            ),
                        )
                    )
                    idx += 3
                    continue
            newoutvec.append(outvec[idx])
            idx += 1
        outvec = newoutvec
        # super
        newoutvec = []
        idx = 0
        while idx < len(outvec):
            if idx + 1 < len(outvec):
                if outvec[idx + 1][0] == 'oper' and outvec[idx + 1][1] == '^':
                    newoutvec.append(
                        (
                            'char',
                            self.__exponent(
                                outvec[idx][1],
                                outvec[idx + 2][1]
                            ),
                        )
                    )
                    idx += 3
                    continue
            newoutvec.append(outvec[idx])
            idx += 1
        outvec = newoutvec
        return ''.join([x[1] for x in outvec])

    def __indexed(self, a, b):
        f_sub = transliterate(data.subscript)
        bsub, ok = f_sub(b)
        if self.__is_complex_expr(a):
            a = '(' + a + ')'
        if ok:
            return a + bsub
        return a + '[' + b + ']'

    def __exponent(self, a, b):
        f_sup = transliterate(data.superscript)
        bsup, ok = f_sup(b)
        if self.__is_complex_expr(a):
            a = '(' + a + ')'
        if ok:
            return a + bsup
        if self.__is_complex_expr(b):
            b = '(' + b + ')'
        return a + '^' + b

    def __is_complex_expr(self, expr):
        return (len(expr) > 1)

    def __latexfun_comb(self, comb, inputs):
        expr = inputs[0]
        if(len(expr) == 1):
            if self.allow_combinings:
                return expr + comb[0]
        return comb[1] + '(' + expr + ')'

    def add_newcommand(self, one_newcommand):
        """Add a command definiton using LaTeX syntax.

        :param expr: a valid \\newcommand (or \\renewcommand or \\def)
            definition.
            Examples:
            - r'\\newcommand\\prob{\\mathbb{P}}'
            - r'\\newcommand\\binom[2]{\\frac{#2!}{#1!(#2-#1)!}}'
        """

        parsed = parser.parse(one_newcommand)
        if not (len(parsed) in (3, 6)):
            raise LatexSyntaxError
        ok = False
        if parsed[0][0] == 'cmd':
            if parsed[0][1] in (
                    r'\newcommand',
                    r'\renewcommand',
                    r'\def'):
                ok = True
        if not ok:
            raise LatexSyntaxError
        nargs = 0
        if len(parsed) == 6:
            if parsed[2] == ('char', '[') and parsed[4] == ('char', ']'):
                nargs = int(parsed[3][1])
            else:
                raise LatexSyntaxError
        cmdname = parsed[1][1]
        cmdexpr = parsed[-1][1]

        def thefun(args):
            expr = cmdexpr
            for i in range(len(args)):
                expr = regex.sub('#%i' % (i + 1), args[i], expr)
            return self.convert(expr)

        self.__cmds[cmdname] = latexfuntypes.latexfun(
            lambda x: thefun(x),
            nargs)
        return None

    def __latexfun_frac(self, inputs):
        a = inputs[0]
        b = inputs[1]

        try:
            ret = data.known_fracts[(a, b)]
            return ret
        except KeyError:
            pass

        if self.allow_zw:
            f_sub = transliterate(data.subscript)
            f_sup = transliterate(data.superscript)

            a_sup, ok_sup = f_sup(a)
            b_sub, ok_sub = f_sub(b)

            if ok_sup and ok_sub:
                return a_sup + '⁄' + b_sub

        if self.__is_complex_expr(a):
            a = '(' + a + ')'
        if self.__is_complex_expr(b):
            b = '(' + b + ')'
        return a + '/' + b

    def __latexfun_sqrt(self, inputs):
        a = inputs[0]

        if self.__is_complex_expr(a):
            a = '(' + a + ')'
        return '√' + a



"""Setup script for flatlatex

Based on `sampleproject`, see:
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the __init__.py file

###
# c'est toi qui extrait la doc comme ca ? sinon to peux exec' the module et
# garder le `__doc__`, tu peux en profiter pour attraper le `__version__` en
# meme temps.
###
with open(path.join(here, 'flatlatex', '__init__.py'), encoding='utf-8') as f:
    ast = compile(f.read(), '__init__.py', 'exec')
    fake_global = {'__name__': '__main__'}
    try:
        exec(ast, fake_global)
    except (SystemError, ImportError) as e:
        print('System error')

    long_description = fake_global['__doc__']
    version = fake_global['__version__']


setup(
    name='flatlatex',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=version,

    description='A LaTeX math converter to unicode text',
    long_description=long_description,

    # The project's main homepage.
    url='https://gitlab.crans.org/leger/flatlatex',

    # Author details
    author='Jean-Benoist Leger',
    author_email='jb@leger.tf',

    # Choose your license
    license='BSD-2',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Science/Research',
        'Topic :: Text Processing :: Markup :: LaTeX',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3 :: Only',
    ],

    # What does your project relate to?
    keywords='latex math unicode',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['regex'],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        'test': ['pytest'],
    },

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data={
    },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    data_files=[],

    python_requires='>=3',

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [
            "flatlatex=flatlatex.bin.flatlatex:main",
        ],
    },
)



# Copyright (c) 2017-2017, Jean-Benoist Leger <jb@leger.tf>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.


from .. import data
from .. import parser

import ast


def test_known_fracts():
    assert type(data.known_fracts) is dict
    for k, v in data.known_fracts.items():
        assert type(k) is tuple
        assert len(k) == 2
        assert all([type(x) is str for x in k])
        assert type(v) is str


def test_transliteration_consistancy():
    for d in (
            data.subscript,
            data.superscript,
            data.bb,
            data.bf,
            data.cal,
            data.frak,
            data.it,
            data.mono,
            ):
        assert type(d) is dict
        for k, v in d.items():
            assert type(k) is str
            assert type(v) is str


def test_symbols_consistancy():
    assert type(data.symbols) is dict
    for k, v in data.symbols.items():
        assert type(k) is str
        out = parser.parse(k)
        assert len(out) == 1
        assert out[0][0] == 'cmd'
        assert type(v) is str


def test_combining_consistancy():
    assert type(data.combinings) is dict
    for k, v in data.combinings.items():
        assert type(k) is str
        out = parser.parse(k)
        assert len(out) == 1
        assert out[0][0] == 'cmd'
        assert type(v) is tuple
        assert len(v) == 2
        assert all([type(x) is str for x in v])


def test_newcommands_consistancy():
    assert type(data.newcommands) is tuple
    for k in data.newcommands:
        parsed = parser.parse(k)
        assert len(parsed) in (3, 6)
        assert parsed[0][0] == 'cmd'
        assert parsed[0][1] in (r'\newcommand', r'\renewcommand', r'\def')
        if len(parsed) == 6:
            assert parsed[2] == ('char', '[')
            assert parsed[4] == ('char', ']')
            assert type(parsed[3][0]) is str
            a = int(parsed[3][1])
            assert a >= 0


def test_replicated_command():
    datasets = [
            data.symbols.keys(),
            data.combinings.keys(),
            data.transliterators.keys(),
    ]
    datasets.append([parser.parse(nc)[1][1] for nc in data.newcommands])
    for i in range(len(datasets)):
        for j in range(i + 1, len(datasets)):
            s1 = set(datasets[i])
            s2 = set(datasets[j])
            assert len(s1.intersection(s2)) == 0


def test_replicated_in_the_same_dict():
    with open(data.__file__, encoding='utf8') as f:
        data_ast = ast.parse(f.read())

    def toobj(x):
        if type(x) is ast.Tuple:
            return tuple(toobj(v) for v in x.elts)
        return x.s

    for seg in data_ast.body:
        if type(seg) is ast.Assign:
            if type(seg.value) is ast.Dict:
                keys = [toobj(k) for k in seg.value.keys]
                count = {}
                for k in keys:
                    count[k] = count[k] + 1 if k in count else 1
                for k, c in count.items():
                    assert c == 1, "duplicated key in dict: %s" % k



# Copyright (c) 2016, Jean-Benoist Leger <jb@leger.tf>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
"""
LaTeX math to Unicode text converter
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

flatlatex is a basic converter from LaTeX math to human readable text math
using unicode characters.

Basic example:

    >>> import flatlatex
    >>> c = flatlatex.converter()
    >>> c.convert(
    ... (
    ...     r'\\forall \\eta>0\\, \\exists n\\in\\mathbb{N}\\, \\forall i>n\\,'
    ...     r' |u_i-\\mathcal{l}|<\\eta'
    ... ))
    '∀η>0 ∃n∈ℕ ∀i>n |uᵢ-𝓵|<η'

Commands can be added with LaTeX syntax:

    >>> import flatlatex
    >>> c = flatlatex.converter()
    >>> c.add_newcommand(r'\\newcommand\\prob{\\mathbb{P}}')
    >>> c.add_newcommand(r'\\newcommand\\binom[2]{\\frac{#2!}{#1!(#2-#1)!}}')
    >>> c.convert(r'\\prob(X=k)\\,=\\,\\binom{k}{n}\\times p^k(1-p)^{n-k}')
    'ℙ(X=k) = (n!)/(k!(n-k)!)×pᵏ(1-p)ⁿ⁻ᵏ'

The behavior can be change:

    >>> import flatlatex
    >>> c = flatlatex.converter()
    >>> c.convert(r'\\frac{8}{9}')
    '⁸⁄₉'
    >>> c.allow_zw = False
    >>> c.convert(r'\\frac{8}{9}')
    '8/9'
"""

__title__ = 'flatlatex'
__author__ = 'Jean-Benoist Leger'
__licence__ = 'BSD-2'

version_info = (0, 8)
__version__ = '.'.join(map(str, version_info))

from .conv import converter
